let React = require("react");
let abi = require("augur-abi");
let BigNumber = require("bignumber.js");
let FluxMixin = require("fluxxor/lib/flux_mixin")(React);
let StoreWatchMixin = require("fluxxor/lib/store_watch_mixin");
let ReportFillForm = require("./ReportFillForm.jsx");
let ReportSavedModal = require("./ReportSavedModal.jsx");
let ReportDetails = require("./ReportDetails.jsx");
let utilities = require("../../libs/utilities");
let constants = require("../../libs/constants");

let ReportPage = React.createClass({
    mixins: [FluxMixin, StoreWatchMixin("branch", "market", "config", "report", "network")],

    getInitialState() {
        return {reportError: null};
    },

    getStateFromFlux() {
        let flux = this.getFlux();
        let account = flux.store("config").getAccount();
        let branch = flux.store("branch").getCurrentBranch();
        let blockNumber = flux.store("network").getState().blockNumber;
        let eventId;
        if (this.props.params && this.props.params.eventId) {
            eventId = this.props.params.eventId;
        } else {
            let pathname = window.location.pathname.split("/");
            eventId = pathname[pathname.length - 1];
        }
        let event = flux.store("report").getEvent(eventId);
        let market, reportedOutcome, isUnethical, report, isIndeterminate;
        if (event && event.markets && event.markets.length) {
            market = event.markets[0];
            report = flux.store("report").getReport(branch.id, branch.reportPeriod, eventId);
            if (report && report.reportedOutcome !== null && report.reportedOutcome !== undefined) {
                reportedOutcome = event.report.reportedOutcome;
                isUnethical = event.report.isUnethical;
                isIndeterminate = event.report.isIndeterminate;
            }
        }
        if (market) {
            if (branch && market.tradingPeriod &&
                branch.currentPeriod >= market.tradingPeriod.toNumber()) {
                market.matured = true;
            }
        }
        return {
            event,
            account,
            market,
            branch,
            blockNumber,
            report,
            reportedOutcome,
            isUnethical,
            isIndeterminate
        };
    },

    onReportFormSubmit(reportedOutcome, isUnethical, isIndeterminate) {
        let self = this;
        let flux = this.getFlux();
        if (reportedOutcome === null) {
            return this.setState({reportError: "you must choose something"});
        }
        this.setState({
            reportError: null,
            reportedOutcome: reportedOutcome,
            isUnethical: isUnethical,
            isIndeterminate: isIndeterminate
        });
        flux.actions.report.submitReportHash(
            this.state.branch.id,
            this.state.event,
            this.state.branch.reportPeriod,
            reportedOutcome,
            isUnethical,
            isIndeterminate,
            function (err, res) {
                if (err) {
                    if (err.error === "-5") {
                        console.log(err);
                        flux.actions.report.loadEventsToReport();
                    } else {
                        console.error("submitReportHash failed:", err);
                    }
                    return self.setState({reportedOutcome: null});
                }
                console.log("submitReportHash complete:", res);
            }
        );
        this.props.toggleReportSavedModal();
    },

    onReportedOutcomeChanged(event) {
        let report = event.target.value;
        if (report !== null && report !== undefined) {
            if (report === constants.INDETERMINATE_OUTCOME && event.target.id === "indeterminate") {
                this.setState({isIndeterminate: true});
            }
            this.setState({reportedOutcome: report});
        }
    },

    _onUnethicalChange(event) {
        this.setState({isUnethical: event.target.checked});
    },

    render() {
        if (this.state.account == null) {
            return <div>Only for logged-in users</div>;
        }
        let market = this.state.market;
        if (market == null) {
            return <div>No market found for this event</div>;
        }
        let blockNumber = this.state.blockNumber;
        let isReportCommitPeriod = this.getFlux().store("branch").isReportCommitPeriod(blockNumber);
        let isReportRevealPeriod = !isReportCommitPeriod;
        let reportedOutcomeName = "";
        if (market.type === "scalar") {
            reportedOutcomeName = this.state.reportedOutcome;
        } else {
            reportedOutcomeName = this.state.reportedOutcome != null ? utilities.getOutcomeName(this.state.reportedOutcome, market).outcome : "none";
        }
        if (market.matured) {
            if (isReportCommitPeriod) {
                return (
                    <div>
                        <h1>Reporting the outcome of a market</h1>
                        <h2>Review the question and make your report</h2>
                        <ReportFillForm onReportFormSubmit={this.onReportFormSubmit}
                            onUnethicalChange={this._onUnethicalChange}
                            onReportedOutcomeChanged={this.onReportedOutcomeChanged}
                            reportedOutcome={this.state.reportedOutcome}
                            isUnethical={this.state.isUnethical}
                            reportError={this.state.reportError}
                            report={this.state.report}
                            market={market} />
                        <ReportDetails market={market} />
                        <ReportSavedModal
                            reportedOutcomeName={reportedOutcomeName}
                            report={this.state.report}
                            isUnethical={this.state.isUnethical}
                            show={this.props.reportSavedModalOpen}
                            onHide={this.props.toggleReportSavedModal} />
                    </div>
                );
            } else {
                return (
                    <div>
                        <h1>Reporting details</h1>
                        <ReportDetails market={market} />
                    </div>
                );
            }
        } else {
            return (
                <div>
                    <h1>Event not ready for Reporting</h1>
                    <ReportDetails market={market} />
                </div>
            );
        }
    }
});

module.exports = ReportPage;
